//----------------------------------------------------------------------------
//
// DETL - A template library 1.0 beta
//
// Douglas Eleveld (D.J.Eleveld@anest.azg.nl)
//
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// Templated vector class
//
// Types used in these arrays must have a default constructor and a sensible
// operator=
//
//----------------------------------------------------------------------------
#ifndef TEMPLATE_VECTOR
#define TEMPLATE_VECTOR

//----------------------------------------------------------------------------
#include "detl.h"
#include "tarray.h"
#include "tcontain.h"

//----------------------------------------------------------------------------
// Default chunk size to allocate
static const int default_chunk = 10;

template <class T> class Tvector
         :public Tarray<T>
         {
         private:
               int _number;
               int _chunk;

               // Exactly copy another array
               void clone (const Tvector<T> &other) { Tarray<T>::clone(other); _number = other._number; _chunk = other._chunk; };

         public:
               // Constructor and destructors
               Tvector (void) : _number(0), _chunk(default_chunk) { };
               Tvector (const int i) : Tarray<T>(i), _number(0), _chunk(default_chunk) { };
               Tvector (const Tvector<T> &other) { clone(other); };
               virtual ~Tvector (void) { };

               Tvector<T>& operator=(const Tvector<T>& other) { clone(other); return *this; };

               // Information functions
               inline bool is_empty  (void) const  { if(_number==0) return true; else return false; };
               inline int  number    (void) const  { return _number;     };
               inline int  chunk     (void) const  { return _chunk;      };

               // Allocation manipulation functions
               inline void set_chunk (const int i) { if(i>0) _chunk = i; };
               inline void clear  (void) { Tarray<T>::clear(); _number = 0; };

               T* first_that (const Tcompare_function<T>&, void*) const;
               T* last_that  (const Tcompare_function<T>&, void*) const;

               // Basic manipulation operations
                      void append (const T&);
               inline void remove (void)           { if(_number>0) _number--; };
                      void remove (const int);
               inline void resize (const int i)    { Tarray<T>::resize(i); _number = i; };

               inline void for_each (const Titer_function<T>& func, void *args) const { for(int i=0;i<_number;i++) func(_data[i],args); };
         };
//----------------------------------------------------------------------------
// Add an element to the end of the Tarray
template <class T> void Tvector<T>::append (const T& elem)
   {
   // Check to see if we have room to add one
   if(_number>=size())
      {
      // No, so make room
      Tarray<T>::resize(size()+_chunk);
      }
   // Set the newly added stuff in
   _data[_number] = elem;
   _number++;
   }
//----------------------------------------------------------------------------
// Remove an element from the list
template <class T> void Tvector<T>::remove (const int index)
   {
   // Check for reasonable index
   if((index>=_number)||(index<0)) return;

   // Check to see if we can contract
   if(_number<(Tarray<T>::size()-_chunk))
      {
      // Yes so contract
      Tarray<T>::resize(_number);
      }
   // Move the later ones down
   for(int i=index;i<_number;i++) (*this)[i] = (*this)[i+1];

   // Take off the redundant end
   remove();
   }
//----------------------------------------------------------------------------
// Returns the first Node in the list which the given function returns an int != 0
template <class T> T* Tvector<T>::first_that (const Tcompare_function<T>& match, void *args) const
   {
   for(int i=0;i<_number;i++)
      if(match(_data[i],args))
         return &(_data[i]);
   return 0;
   }
//----------------------------------------------------------------------------
// Returns the last Node in the list which the given function returns an int != 0
template <class T> T* Tvector<T>::last_that (const Tcompare_function<T>& match, void *args) const
   {
   for(int i=_number-1;i>0;i--)
      if(match(_data[i],args))
         return &(_data[i]);
   return 0;
   }
//----------------------------------------------------------------------------
#endif
